;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Invincible Shell 
; By Sonikku
; 
; Description: A shell that flashes and follows the player, 
; much like state shells are in when Yellow Shellless Koopas
; hop into a shell. This also gives Yoshi the Fire, Flying, and 
; Ground Pound abilities when eaten. The only difference between
; this and the original is that no Yellow Shellless Koopas 
; need to hop into this one for it to flash, as it does it automatically.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; init JSL and main
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	dcb "INIT"
	RTL

	dcb "MAIN"
	PHB
	PHK
	PLB
	JSR SPRITEMAIN
	PLB
	RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; main sprite routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PROP		dcb $00,$00,$00,$40
TILEMAP		dcb $8C,$8A,$8E,$8A
SPRITEMAIN
	JSR SUB_OFF_SCREEN_X0
	JSL $0190B2	; load generic graphics.
	LDY $15EA,x	; load oam..
	PHX		; push x..
	LDA $1570,x	; display a new frame..
	AND #$0C	; every #$0C frames.
	LSR		; divide..
	ADC $15E9	; by sprite index for sprite..
	LSR		; divide again by..
	AND #$03	; number of frames -1.
	TAX		; transfer a to x.
	LDA TILEMAP,x	; load tilemap..
	STA $0302,y	; store tilemap.

	PHX		; push x again.
	LDX $15E9	; x = sprite index for sprite..

	LDA $15F6,x	; load palette properties..

	PLX		; push x..
	ORA PROP,x	; and load flip..
	ORA $64		; and priority bits..
	STA $0303,y	; and store it.

	LDA #$F0	; delete the tile behind the shell..

	STA $0309,y	; since its annoying.
	PLX		; pull x.
	LDA $14C8,x	; if status..
	CMP #$08	; is 8 or less..
	BCC DEAD	; its dead.
	LDA #$01	; if not..
	STA $187B,x	; it is flashing and invincible.
	INC $1570,x	; increase $1570 (for frame count)..
	LDA $15D0,x	; if being eaten..
	BNE RETURN	; return.
	JSL $01A7DC	; interact with mario.
	LDA #$0A	; sprite status is always 0A.

	STA $14C8,x	; so it always flashes except when dead.
RETURN	RTS		; return.
DEAD	LDA $15F6,x	; flip the..
	ORA #$80	; shell..
	STA $15F6,x	; vertically.
	RTS		; return
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; off screen processing code - shared
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;org $03B83B             

TABLE3              dcb $40,$B0
TABLE6              dcb $01,$FF 
TABLE4              dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0 
TABLE5              dcb $01,$FF,$01,$FF,$01,$00,$01,$FF

SUB_OFF_SCREEN_X0   LDA #$06                ; \ entry point of routine determines value of $03
	BRA STORE_03            ; | 
SUB_OFF_SCREEN_X1   LDA #$04                ; |
	BRA STORE_03            ; |
SUB_OFF_SCREEN_X2   LDA #$02                ; |
STORE_03            STA $03                 ; |
	BRA START_SUB           ; |
SUB_OFF_SCREEN_X3  STZ $03                 ; /

START_SUB           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
	BEQ RETURN_2            ; /    
	LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
	AND #$01                ; |
	BNE VERTICAL_LEVEL      ; /     
	LDA $D8,x               ; \
	CLC	 ; | 
	ADC #$50                ; | if the sprite has gone off the bottom of the level...
	LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
	ADC #$00                ; | 
	CMP #$02                ; | 
	BPL ERASE_SPRITE        ; /    ...erase the sprite
	LDA $167A,x             ; \ if "process offscreen" flag is set, return
	AND #$04                ; |
	BNE RETURN_2            ; /
	LDA $13                 ; \ 
	AND #$01                ; | 
	ORA $03                 ; | 
	STA $01                 ; |
	TAY	 ; /
	LDA $1A
	CLC
	ADC TABLE4,y
	ROL $00
	CMP $E4,x
	PHP
	LDA $1B
	LSR $00
	ADC TABLE5,y
	PLP
	SBC $14E0,x
	STA $00
	LSR $01
	BCC LABEL20
	EOR #$80
	STA $00
LABEL20             LDA $00
	BPL RETURN_2
ERASE_SPRITE        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
	CMP #$08                ; |
	BCC KILL_SPRITE         ; /
	LDY $161A,x
	CPY #$FF
	BEQ KILL_SPRITE
	LDA #$00
	STA $1938,y
KILL_SPRITE         STZ $14C8,x             ; erase sprite
RETURN_2            RTS	 ; return

VERTICAL_LEVEL      LDA $167A,x             ; \ if "process offscreen" flag is set, return
	AND #$04                ; |
	BNE RETURN_2            ; /
	LDA $13                 ; \ only handle every other frame??
	LSR A                   ; | 
	BCS RETURN_2            ; /
	AND #$01
	STA $01
	TAY
	LDA $1C
	CLC
	ADC TABLE3,y
	ROL $00
	CMP $D8,x
	PHP
	LDA.W $001D
	LSR $00
	ADC TABLE6,y
	PLP
	SBC $14D4,x
	STA $00
	LDY $01
	BEQ LABEL22
	EOR #$80
	STA $00
LABEL22             LDA $00
	BPL RETURN_2
	BMI ERASE_SPRITE

SUB_IS_OFF_SCREEN   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
	ORA $186C,x             ; |  
	RTS	 ; / return